@@ -75,10 +75,10 @@ extern "C" {
 #define TK_MAJOR_VERSION	8
 #define TK_MINOR_VERSION	6
 #define TK_RELEASE_LEVEL	TCL_FINAL_RELEASE
-#define TK_RELEASE_SERIAL	13
+#define TK_RELEASE_SERIAL	17
 
 #define TK_VERSION		"8.6"
-#define TK_PATCH_LEVEL		"8.6.13"
+#define TK_PATCH_LEVEL		"8.6.17"
 
 /*
  * A special definition used to allow this header file to be included from
@@ -117,7 +117,7 @@ extern "C" {
  * Decide whether or not to use input methods.
  */
 
-#ifdef XNQueryInputStyle
+#if defined(XNQueryInputStyle) && !defined(_WIN32) && !defined(MAC_OSX_TK)
 #define TK_USE_INPUT_METHODS
 #endif
 
