@@ -4,8 +4,7 @@
  * Ttk package: initialization routine and miscellaneous utilities.
  */
 
-#include <string.h>
-#include <tk.h>
+#include "tkInt.h"
 #include "ttkTheme.h"
 #include "ttkWidget.h"
 
@@ -13,7 +12,7 @@
  * Legal values for the button -default option.
  * See also: enum Ttk_ButtonDefaultState.
  */
-const char *ttkDefaultStrings[] = {
+const char *const ttkDefaultStrings[] = {
     "normal", "active", "disabled", NULL
 };
 
@@ -29,7 +28,7 @@ int Ttk_GetButtonDefaultStateFromObj(
  * Legal values for the -compound option.
  * See also: enum Ttk_Compound.
  */
-const char *ttkCompoundStrings[] = {
+const char *const ttkCompoundStrings[] = {
     "none", "text", "image", "center",
     "top", "bottom", "left", "right", NULL
 };
@@ -46,7 +45,7 @@ int Ttk_GetCompoundFromObj(
  * Legal values for the -orient option.
  * See also: enum Ttk_Orient.
  */
-const char *ttkOrientStrings[] = {
+const char *const ttkOrientStrings[] = {
     "horizontal", "vertical", NULL
 };
 
@@ -62,7 +61,7 @@ int Ttk_GetOrientFromObj(
  * Recognized values for the -state compatibility option.
  * Other options are accepted and interpreted as synonyms for "normal".
  */
-static const char *ttkStateStrings[] = {
+static const char *const ttkStateStrings[] = {
     "normal", "readonly", "disabled", "active", NULL
 };
 enum {
@@ -108,14 +107,14 @@ void TtkCheckStateOption(WidgetCore *corePtr, Tcl_Obj *objPtr)
 
 /* TtkSendVirtualEvent --
  * 	Send a virtual event notification to the specified target window.
- * 	Equivalent to "event generate $tgtWindow <<$eventName>>"
+ * 	Equivalent to "event generate $tgtWindow <<$eventName>> -when tail"
  *
  * 	Note that we use Tk_QueueWindowEvent, not Tk_HandleEvent,
  * 	so this routine does not reenter the interpreter.
  */
 void TtkSendVirtualEvent(Tk_Window tgtWin, const char *eventName)
 {
-    union {XEvent general; XVirtualEvent virtual;} event;
+    union {XEvent general; XVirtualEvent virt;} event;
 
     memset(&event, 0, sizeof(event));
     event.general.xany.type = VirtualEvent;
@@ -123,7 +122,7 @@ void TtkSendVirtualEvent(Tk_Window tgtWin, const char *eventName)
     event.general.xany.send_event = False;
     event.general.xany.window = Tk_WindowId(tgtWin);
     event.general.xany.display = Tk_Display(tgtWin);
-    event.virtual.name = Tk_GetUid(eventName);
+    event.virt.name = Tk_GetUid(eventName);
 
     Tk_QueueWindowEvent(&event.general, TCL_QUEUE_TAIL);
 }
@@ -275,7 +274,7 @@ Ttk_Init(Tcl_Interp *interp)
 
     Ttk_PlatformInit(interp);
 
-    Tcl_PkgProvideEx(interp, "Ttk", TTK_PATCH_LEVEL, (ClientData)&ttkStubs);
+    Tcl_PkgProvideEx(interp, "Ttk", TTK_PATCH_LEVEL, (void *)&ttkStubs);
 
     return TCL_OK;
 }
