@@ -7,7 +7,7 @@
  *
  * Scrollable interface:
  *
- * 	+ 'first' is controlled by [xy]view widget command 
+ * 	+ 'first' is controlled by [xy]view widget command
  * 	  and other scrolling commands like 'see';
  *      + 'total' depends on widget contents;
  *      + 'last' depends on first, total, and widget size.
@@ -16,15 +16,15 @@
  *
  * 	1. User adjusts scrollbar, scrollbar widget calls its -command
  * 	2. Scrollbar -command invokes the scrollee [xy]view widget method
- * 	3. TtkScrollviewCommand calls TtkScrollTo(), which updates 
+ * 	3. TtkScrollviewCommand calls TtkScrollTo(), which updates
  * 	   'first' and schedules a redisplay.
- * 	4. Once the scrollee knows 'total' and 'last' (typically in 
- * 	   the LayoutProc), call TtkScrolled(h,first,last,total) to 
+ * 	4. Once the scrollee knows 'total' and 'last' (typically in
+ * 	   the LayoutProc), call TtkScrolled(h,first,last,total) to
  * 	   synchronize the scrollbar.
  * 	5. The scrollee -[xy]scrollcommand is called (in an idle callback)
  * 	6. Which calls the scrollbar 'set' method and redisplays the scrollbar.
  *
- * If the scrollee has internal scrolling (e.g., a 'see' method), 
+ * If the scrollee has internal scrolling (e.g., a 'see' method),
  * it should TtkScrollTo() directly (step 2).
  *
  * If the widget value changes, it should call TtkScrolled() (step 4).
@@ -34,7 +34,7 @@
  * TtkScrollbarUpdateRequired, which will invoke step (5) (@@@ Fix this)
  */
 
-#include <tk.h>
+#include "tkInt.h"
 #include "ttkTheme.h"
 #include "ttkWidget.h"
 
@@ -78,6 +78,7 @@ static int UpdateScrollbar(Tcl_Interp *interp, ScrollHandle h)
     char arg1[TCL_DOUBLE_SPACE + 2];
     char arg2[TCL_DOUBLE_SPACE + 2];
     int code;
+    Tcl_DString buf;
 
     h->flags &= ~SCROLL_UPDATE_REQUIRED;
 
@@ -88,9 +89,14 @@ static int UpdateScrollbar(Tcl_Interp *interp, ScrollHandle h)
     arg1[0] = arg2[0] = ' ';
     Tcl_PrintDouble(interp, (double)s->first / s->total, arg1+1);
     Tcl_PrintDouble(interp, (double)s->last / s->total, arg2+1);
+    Tcl_DStringInit(&buf);
+    Tcl_DStringAppend(&buf, s->scrollCmd, -1);
+    Tcl_DStringAppend(&buf, arg1, -1);
+    Tcl_DStringAppend(&buf, arg2, -1);
 
     Tcl_Preserve(corePtr);
-    code = Tcl_VarEval(interp, s->scrollCmd, arg1, arg2, NULL);
+    code = Tcl_EvalEx(interp, Tcl_DStringValue(&buf), -1, TCL_EVAL_GLOBAL);
+    Tcl_DStringFree(&buf);
     if (WidgetDestroyed(corePtr)) {
 	Tcl_Release(corePtr);
 	return TCL_ERROR;
@@ -98,15 +104,18 @@ static int UpdateScrollbar(Tcl_Interp *interp, ScrollHandle h)
     Tcl_Release(corePtr);
 
     if (code != TCL_OK && !Tcl_InterpDeleted(interp)) {
-	/* Disable the -scrollcommand, add to stack trace:
+	/* Add error to stack trace.
+         * Also set the SCROLL_UPDATE_REQUIRED flag so that a later call to
+         * TtkScrolled has an effect. Indeed, the error in the -scrollcommand
+         * callback may later be gone, for instance the callback proc got
+         * defined in the meantime.
 	 */
-	ckfree(s->scrollCmd);
-	s->scrollCmd = 0;
 
 	Tcl_AddErrorInfo(interp, /* @@@ "horizontal" / "vertical" */
 		"\n    (scrolling command executed by ");
 	Tcl_AddErrorInfo(interp, Tk_PathName(h->corePtr->tkwin));
 	Tcl_AddErrorInfo(interp, ")");
+        TtkScrollbarUpdateRequired(h);
     }
     return code;
 }
@@ -124,7 +133,7 @@ static void UpdateScrollbarBG(ClientData clientData)
     Tcl_Preserve((ClientData) interp);
     code = UpdateScrollbar(interp, h);
     if (code == TCL_ERROR && !Tcl_InterpDeleted(interp)) {
-	Tcl_BackgroundError(interp);
+	Tcl_BackgroundException(interp, code);
     }
     Tcl_Release((ClientData) interp);
 }
@@ -135,7 +144,7 @@ static void UpdateScrollbarBG(ClientData clientData)
 void TtkScrolled(ScrollHandle h, int first, int last, int total)
 {
     Scrollable *s = h->scrollPtr;
-    
+
     /* Sanity-check inputs:
      */
     if (total <= 0) {
@@ -175,6 +184,19 @@ void TtkScrollbarUpdateRequired(ScrollHandle h)
     h->flags |= SCROLL_UPDATE_REQUIRED;
 }
 
+/* TtkUpdateScrollInfo --
+ * 	Call the layoutProc to update the scroll info first, last, and total.
+ * 	Do it only if needed, that is when a redisplay is pending (which
+ * 	indicates scroll info are possibly out of date).
+ */
+
+void TtkUpdateScrollInfo(ScrollHandle h)
+{
+    if (h->corePtr->flags & REDISPLAY_PENDING) {
+        h->corePtr->widgetSpec->layoutProc(h->corePtr);
+    }
+}
+
 /* TtkScrollviewCommand --
  * 	Widget [xy]view command implementation.
  *
@@ -187,7 +209,10 @@ int TtkScrollviewCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], ScrollHandle h)
 {
     Scrollable *s = h->scrollPtr;
-    int newFirst = s->first;
+    int newFirst;
+
+    TtkUpdateScrollInfo(h);
+    newFirst = s->first;
 
     if (objc == 2) {
 	Tcl_Obj *result[2];
@@ -204,8 +229,6 @@ int TtkScrollviewCommand(
 	int count;
 
 	switch (Tk_GetScrollInfoObj(interp, objc, objv, &fraction, &count)) {
-	    case TK_SCROLL_ERROR:
-		return TCL_ERROR;
 	    case TK_SCROLL_MOVETO:
 		newFirst = (int) ((fraction * s->total) + 0.5);
 		break;
@@ -217,18 +240,24 @@ int TtkScrollviewCommand(
 		newFirst = s->first + count * perPage;
 		break;
 	    }
+	    default:
+		return TCL_ERROR;
 	}
     }
 
-    TtkScrollTo(h, newFirst);
+    TtkScrollTo(h, newFirst, 0);
 
     return TCL_OK;
 }
 
-void TtkScrollTo(ScrollHandle h, int newFirst)
+void TtkScrollTo(ScrollHandle h, int newFirst, int updateScrollInfo)
 {
     Scrollable *s = h->scrollPtr;
 
+    if (updateScrollInfo) {
+        TtkUpdateScrollInfo(h);
+    }
+
     if (newFirst >= s->total)
 	newFirst = s->total - 1;
     if (newFirst > s->first && s->last >= s->total) /* don't scroll past end */
