@@ -3,9 +3,9 @@
  *
  *	This file contains Macintosh specific cursor related routines.
  *
- * Copyright (c) 1995-1997 Sun Microsystems, Inc.
- * Copyright 2001-2009, Apple Inc.
- * Copyright (c) 2006-2009 Daniel A. Steffen <das@users.sourceforge.net>
+ * Copyright © 1995-1997 Sun Microsystems, Inc.
+ * Copyright © 2001-2009 Apple Inc.
+ * Copyright © 2006-2009 Daniel A. Steffen <das@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -54,131 +54,131 @@ struct CursorName {
 #define MacXCursorData(n)	((id)tkMacOSXXCursors[TK_MAC_XCURSOR_##n])
 
 static const struct CursorName cursorNames[] = {
-    {"none",			NONE,	    nil},
-    {"arrow",			SELECTOR,    @"arrowCursor"},
-    {"top_left_arrow",		SELECTOR,    @"arrowCursor"},
-    {"left_ptr",		SELECTOR,    @"arrowCursor"},
-    {"copyarrow",		SELECTOR,    @"dragCopyCursor", @"_copyDragCursor"},
-    {"aliasarrow",		SELECTOR,    @"dragLinkCursor", @"_linkDragCursor"},
-    {"contextualmenuarrow",	SELECTOR,    @"contextualMenuCursor"},
-    {"movearrow",		SELECTOR,    @"_moveCursor"},
-    {"ibeam",			SELECTOR,    @"IBeamCursor"},
-    {"text",			SELECTOR,    @"IBeamCursor"},
-    {"xterm",			SELECTOR,    @"IBeamCursor"},
-    {"cross",			SELECTOR,    @"crosshairCursor"},
-    {"crosshair",		SELECTOR,    @"crosshairCursor"},
-    {"cross-hair",		SELECTOR,    @"crosshairCursor"},
-    {"tcross",			SELECTOR,    @"crosshairCursor"},
-    {"hand",			SELECTOR,    @"openHandCursor"},
-    {"openhand",		SELECTOR,    @"openHandCursor"},
-    {"closedhand",		SELECTOR,    @"closedHandCursor"},
-    {"fist",			SELECTOR,    @"closedHandCursor"},
-    {"pointinghand",		SELECTOR,    @"pointingHandCursor"},
-    {"resize",			SELECTOR,    @"arrowCursor"},
-    {"resizeleft",		SELECTOR,    @"resizeLeftCursor"},
-    {"resizeright",		SELECTOR,    @"resizeRightCursor"},
-    {"resizeleftright",		SELECTOR,    @"resizeLeftRightCursor"},
-    {"resizeup",		SELECTOR,    @"resizeUpCursor"},
-    {"resizedown",		SELECTOR,    @"resizeDownCursor"},
-    {"resizeupdown",		SELECTOR,    @"resizeUpDownCursor"},
-    {"resizebottomleft",	SELECTOR,    @"_bottomLeftResizeCursor"},
-    {"resizetopleft",		SELECTOR,    @"_topLeftResizeCursor"},
-    {"resizebottomright",	SELECTOR,    @"_bottomRightResizeCursor"},
-    {"resizetopright",		SELECTOR,    @"_topRightResizeCursor"},
-    {"notallowed",		SELECTOR,    @"operationNotAllowedCursor"},
-    {"poof",			SELECTOR,    @"disappearingItemCursor"},
-    {"wait",			SELECTOR,    @"busyButClickableCursor"},
-    {"spinning",		SELECTOR,    @"busyButClickableCursor"},
-    {"countinguphand",		SELECTOR,    @"busyButClickableCursor"},
-    {"countingdownhand",	SELECTOR,    @"busyButClickableCursor"},
-    {"countingupanddownhand",	SELECTOR,    @"busyButClickableCursor"},
+    {"none",			NONE,	    nil, nil, {0, 0}},
+    {"arrow",			SELECTOR,    @"arrowCursor", nil, {0, 0}},
+    {"top_left_arrow",		SELECTOR,    @"arrowCursor", nil, {0, 0}},
+    {"left_ptr",		SELECTOR,    @"arrowCursor", nil, {0, 0}},
+    {"copyarrow",		SELECTOR,    @"dragCopyCursor", @"_copyDragCursor", {0, 0}},
+    {"aliasarrow",		SELECTOR,    @"dragLinkCursor", @"_linkDragCursor", {0, 0}},
+    {"contextualmenuarrow",	SELECTOR,    @"contextualMenuCursor", nil, {0, 0}},
+    {"movearrow",		SELECTOR,    @"_moveCursor", nil, {0, 0}},
+    {"ibeam",			SELECTOR,    @"IBeamCursor", nil, {0, 0}},
+    {"text",			SELECTOR,    @"IBeamCursor", nil, {0, 0}},
+    {"xterm",			SELECTOR,    @"IBeamCursor", nil, {0, 0}},
+    {"cross",			SELECTOR,    @"crosshairCursor", nil, {0, 0}},
+    {"crosshair",		SELECTOR,    @"crosshairCursor", nil, {0, 0}},
+    {"cross-hair",		SELECTOR,    @"crosshairCursor", nil, {0, 0}},
+    {"tcross",			SELECTOR,    @"crosshairCursor", nil, {0, 0}},
+    {"hand",			SELECTOR,    @"openHandCursor", nil, {0, 0}},
+    {"openhand",		SELECTOR,    @"openHandCursor", nil, {0, 0}},
+    {"closedhand",		SELECTOR,    @"closedHandCursor", nil, {0, 0}},
+    {"fist",			SELECTOR,    @"closedHandCursor", nil, {0, 0}},
+    {"pointinghand",		SELECTOR,    @"pointingHandCursor", nil, {0, 0}},
+    {"resize",			SELECTOR,    @"arrowCursor", nil, {0, 0}},
+    {"resizeleft",		SELECTOR,    @"resizeLeftCursor", nil, {0, 0}},
+    {"resizeright",		SELECTOR,    @"resizeRightCursor", nil, {0, 0}},
+    {"resizeleftright",		SELECTOR,    @"resizeLeftRightCursor", nil, {0, 0}},
+    {"resizeup",		SELECTOR,    @"resizeUpCursor", nil, {0, 0}},
+    {"resizedown",		SELECTOR,    @"resizeDownCursor", nil, {0, 0}},
+    {"resizeupdown",		SELECTOR,    @"resizeUpDownCursor", nil, {0, 0}},
+    {"resizebottomleft",	SELECTOR,    @"_bottomLeftResizeCursor", nil, {0, 0}},
+    {"resizetopleft",		SELECTOR,    @"_topLeftResizeCursor", nil, {0, 0}},
+    {"resizebottomright",	SELECTOR,    @"_bottomRightResizeCursor", nil, {0, 0}},
+    {"resizetopright",		SELECTOR,    @"_topRightResizeCursor", nil, {0, 0}},
+    {"notallowed",		SELECTOR,    @"operationNotAllowedCursor", nil, {0, 0}},
+    {"poof",			SELECTOR,    @"disappearingItemCursor", nil, {0, 0}},
+    {"wait",			SELECTOR,    @"busyButClickableCursor", nil, {0, 0}},
+    {"spinning",		SELECTOR,    @"busyButClickableCursor", nil, {0, 0}},
+    {"countinguphand",		SELECTOR,    @"busyButClickableCursor", nil, {0, 0}},
+    {"countingdownhand",	SELECTOR,    @"busyButClickableCursor", nil, {0, 0}},
+    {"countingupanddownhand",	SELECTOR,    @"busyButClickableCursor", nil, {0, 0}},
     {"help",			IMAGENAMED,  @"NSHelpCursor", nil, {8, 8}},
-//  {"hand",			IMAGEBITMAP, MacCursorData(hand)},
-    {"bucket",			IMAGEBITMAP, MacCursorData(bucket)},
-    {"cancel",			IMAGEBITMAP, MacCursorData(cancel)},
-//  {"resize",			IMAGEBITMAP, MacCursorData(resize)},
-    {"eyedrop",			IMAGEBITMAP, MacCursorData(eyedrop)},
-    {"eyedrop-full",		IMAGEBITMAP, MacCursorData(eyedrop_full)},
-    {"zoom-in",			IMAGEBITMAP, MacCursorData(zoom_in)},
-    {"zoom-out",		IMAGEBITMAP, MacCursorData(zoom_out)},
-    {"X_cursor",		IMAGEBITMAP, MacXCursorData(X_cursor)},
-//  {"arrow",			IMAGEBITMAP, MacXCursorData(arrow)},
-    {"based_arrow_down",	IMAGEBITMAP, MacXCursorData(based_arrow_down)},
-    {"based_arrow_up",		IMAGEBITMAP, MacXCursorData(based_arrow_up)},
-    {"boat",			IMAGEBITMAP, MacXCursorData(boat)},
-    {"bogosity",		IMAGEBITMAP, MacXCursorData(bogosity)},
-    {"bottom_left_corner",	IMAGEBITMAP, MacXCursorData(bottom_left_corner)},
-    {"bottom_right_corner",	IMAGEBITMAP, MacXCursorData(bottom_right_corner)},
-    {"bottom_side",		IMAGEBITMAP, MacXCursorData(bottom_side)},
-    {"bottom_tee",		IMAGEBITMAP, MacXCursorData(bottom_tee)},
-    {"box_spiral",		IMAGEBITMAP, MacXCursorData(box_spiral)},
-    {"center_ptr",		IMAGEBITMAP, MacXCursorData(center_ptr)},
-    {"circle",			IMAGEBITMAP, MacXCursorData(circle)},
-    {"clock",			IMAGEBITMAP, MacXCursorData(clock)},
-    {"coffee_mug",		IMAGEBITMAP, MacXCursorData(coffee_mug)},
-//  {"cross",			IMAGEBITMAP, MacXCursorData(cross)},
-    {"cross_reverse",		IMAGEBITMAP, MacXCursorData(cross_reverse)},
-//  {"crosshair",		IMAGEBITMAP, MacXCursorData(crosshair)},
-    {"diamond_cross",		IMAGEBITMAP, MacXCursorData(diamond_cross)},
-    {"dot",			IMAGEBITMAP, MacXCursorData(dot)},
-    {"dotbox",			IMAGEBITMAP, MacXCursorData(dotbox)},
-    {"double_arrow",		IMAGEBITMAP, MacXCursorData(double_arrow)},
-    {"draft_large",		IMAGEBITMAP, MacXCursorData(draft_large)},
-    {"draft_small",		IMAGEBITMAP, MacXCursorData(draft_small)},
-    {"draped_box",		IMAGEBITMAP, MacXCursorData(draped_box)},
-    {"exchange",		IMAGEBITMAP, MacXCursorData(exchange)},
-    {"fleur",			IMAGEBITMAP, MacXCursorData(fleur)},
-    {"gobbler",			IMAGEBITMAP, MacXCursorData(gobbler)},
-    {"gumby",			IMAGEBITMAP, MacXCursorData(gumby)},
-    {"hand1",			IMAGEBITMAP, MacXCursorData(hand1)},
-    {"hand2",			IMAGEBITMAP, MacXCursorData(hand2)},
-    {"heart",			IMAGEBITMAP, MacXCursorData(heart)},
-    {"icon",			IMAGEBITMAP, MacXCursorData(icon)},
-    {"iron_cross",		IMAGEBITMAP, MacXCursorData(iron_cross)},
-//  {"left_ptr",		IMAGEBITMAP, MacXCursorData(left_ptr)},
-    {"left_side",		IMAGEBITMAP, MacXCursorData(left_side)},
-    {"left_tee",		IMAGEBITMAP, MacXCursorData(left_tee)},
-    {"leftbutton",		IMAGEBITMAP, MacXCursorData(leftbutton)},
-    {"ll_angle",		IMAGEBITMAP, MacXCursorData(ll_angle)},
-    {"lr_angle",		IMAGEBITMAP, MacXCursorData(lr_angle)},
-    {"man",			IMAGEBITMAP, MacXCursorData(man)},
-    {"middlebutton",		IMAGEBITMAP, MacXCursorData(middlebutton)},
-    {"mouse",			IMAGEBITMAP, MacXCursorData(mouse)},
-    {"pencil",			IMAGEBITMAP, MacXCursorData(pencil)},
-    {"pirate",			IMAGEBITMAP, MacXCursorData(pirate)},
-    {"plus",			IMAGEBITMAP, MacXCursorData(plus)},
-    {"question_arrow",		IMAGEBITMAP, MacXCursorData(question_arrow)},
-    {"right_ptr",		IMAGEBITMAP, MacXCursorData(right_ptr)},
-    {"right_side",		IMAGEBITMAP, MacXCursorData(right_side)},
-    {"right_tee",		IMAGEBITMAP, MacXCursorData(right_tee)},
-    {"rightbutton",		IMAGEBITMAP, MacXCursorData(rightbutton)},
-    {"rtl_logo",		IMAGEBITMAP, MacXCursorData(rtl_logo)},
-    {"sailboat",		IMAGEBITMAP, MacXCursorData(sailboat)},
-    {"sb_down_arrow",		IMAGEBITMAP, MacXCursorData(sb_down_arrow)},
-    {"sb_h_double_arrow",	IMAGEBITMAP, MacXCursorData(sb_h_double_arrow)},
-    {"sb_left_arrow",		IMAGEBITMAP, MacXCursorData(sb_left_arrow)},
-    {"sb_right_arrow",		IMAGEBITMAP, MacXCursorData(sb_right_arrow)},
-    {"sb_up_arrow",		IMAGEBITMAP, MacXCursorData(sb_up_arrow)},
-    {"sb_v_double_arrow",	IMAGEBITMAP, MacXCursorData(sb_v_double_arrow)},
-    {"shuttle",			IMAGEBITMAP, MacXCursorData(shuttle)},
-    {"sizing",			IMAGEBITMAP, MacXCursorData(sizing)},
-    {"spider",			IMAGEBITMAP, MacXCursorData(spider)},
-    {"spraycan",		IMAGEBITMAP, MacXCursorData(spraycan)},
-    {"star",			IMAGEBITMAP, MacXCursorData(star)},
-    {"target",			IMAGEBITMAP, MacXCursorData(target)},
-//  {"tcross",			IMAGEBITMAP, MacXCursorData(tcross)},
-//  {"top_left_arrow",		IMAGEBITMAP, MacXCursorData(top_left_arrow)},
-    {"top_left_corner",		IMAGEBITMAP, MacXCursorData(top_left_corner)},
-    {"top_right_corner",	IMAGEBITMAP, MacXCursorData(top_right_corner)},
-    {"top_side",		IMAGEBITMAP, MacXCursorData(top_side)},
-    {"top_tee",			IMAGEBITMAP, MacXCursorData(top_tee)},
-    {"trek",			IMAGEBITMAP, MacXCursorData(trek)},
-    {"ul_angle",		IMAGEBITMAP, MacXCursorData(ul_angle)},
-    {"umbrella",		IMAGEBITMAP, MacXCursorData(umbrella)},
-    {"ur_angle",		IMAGEBITMAP, MacXCursorData(ur_angle)},
-    {"watch",			IMAGEBITMAP, MacXCursorData(watch)},
-//  {"xterm",			IMAGEBITMAP, MacXCursorData(xterm)},
-    {NULL}
+//  {"hand",			IMAGEBITMAP, MacCursorData(hand), nil, {0, 0}},
+    {"bucket",			IMAGEBITMAP, MacCursorData(bucket), nil, {0, 0}},
+    {"cancel",			IMAGEBITMAP, MacCursorData(cancel), nil, {0, 0}},
+//  {"resize",			IMAGEBITMAP, MacCursorData(resize), nil, {0, 0}},
+    {"eyedrop",			IMAGEBITMAP, MacCursorData(eyedrop), nil, {0, 0}},
+    {"eyedrop-full",		IMAGEBITMAP, MacCursorData(eyedrop_full), nil, {0, 0}},
+    {"zoom-in",			IMAGEBITMAP, MacCursorData(zoom_in), nil, {0, 0}},
+    {"zoom-out",		IMAGEBITMAP, MacCursorData(zoom_out), nil, {0, 0}},
+    {"X_cursor",		IMAGEBITMAP, MacXCursorData(X_cursor), nil, {0, 0}},
+//  {"arrow",			IMAGEBITMAP, MacXCursorData(arrow), nil, {0, 0}},
+    {"based_arrow_down",	IMAGEBITMAP, MacXCursorData(based_arrow_down), nil, {0, 0}},
+    {"based_arrow_up",		IMAGEBITMAP, MacXCursorData(based_arrow_up), nil, {0, 0}},
+    {"boat",			IMAGEBITMAP, MacXCursorData(boat), nil, {0, 0}},
+    {"bogosity",		IMAGEBITMAP, MacXCursorData(bogosity), nil, {0, 0}},
+    {"bottom_left_corner",	IMAGEBITMAP, MacXCursorData(bottom_left_corner), nil, {0, 0}},
+    {"bottom_right_corner",	IMAGEBITMAP, MacXCursorData(bottom_right_corner), nil, {0, 0}},
+    {"bottom_side",		IMAGEBITMAP, MacXCursorData(bottom_side), nil, {0, 0}},
+    {"bottom_tee",		IMAGEBITMAP, MacXCursorData(bottom_tee), nil, {0, 0}},
+    {"box_spiral",		IMAGEBITMAP, MacXCursorData(box_spiral), nil, {0, 0}},
+    {"center_ptr",		IMAGEBITMAP, MacXCursorData(center_ptr), nil, {0, 0}},
+    {"circle",			IMAGEBITMAP, MacXCursorData(circle), nil, {0, 0}},
+    {"clock",			IMAGEBITMAP, MacXCursorData(clock), nil, {0, 0}},
+    {"coffee_mug",		IMAGEBITMAP, MacXCursorData(coffee_mug), nil, {0, 0}},
+//  {"cross",			IMAGEBITMAP, MacXCursorData(cross), nil, {0, 0}},
+    {"cross_reverse",		IMAGEBITMAP, MacXCursorData(cross_reverse), nil, {0, 0}},
+//  {"crosshair",		IMAGEBITMAP, MacXCursorData(crosshair), nil, {0, 0}},
+    {"diamond_cross",		IMAGEBITMAP, MacXCursorData(diamond_cross), nil, {0, 0}},
+    {"dot",			IMAGEBITMAP, MacXCursorData(dot), nil, {0, 0}},
+    {"dotbox",			IMAGEBITMAP, MacXCursorData(dotbox), nil, {0, 0}},
+    {"double_arrow",		IMAGEBITMAP, MacXCursorData(double_arrow), nil, {0, 0}},
+    {"draft_large",		IMAGEBITMAP, MacXCursorData(draft_large), nil, {0, 0}},
+    {"draft_small",		IMAGEBITMAP, MacXCursorData(draft_small), nil, {0, 0}},
+    {"draped_box",		IMAGEBITMAP, MacXCursorData(draped_box), nil, {0, 0}},
+    {"exchange",		IMAGEBITMAP, MacXCursorData(exchange), nil, {0, 0}},
+    {"fleur",			IMAGEBITMAP, MacXCursorData(fleur), nil, {0, 0}},
+    {"gobbler",			IMAGEBITMAP, MacXCursorData(gobbler), nil, {0, 0}},
+    {"gumby",			IMAGEBITMAP, MacXCursorData(gumby), nil, {0, 0}},
+    {"hand1",			IMAGEBITMAP, MacXCursorData(hand1), nil, {0, 0}},
+    {"hand2",			IMAGEBITMAP, MacXCursorData(hand2), nil, {0, 0}},
+    {"heart",			IMAGEBITMAP, MacXCursorData(heart), nil, {0, 0}},
+    {"icon",			IMAGEBITMAP, MacXCursorData(icon), nil, {0, 0}},
+    {"iron_cross",		IMAGEBITMAP, MacXCursorData(iron_cross), nil, {0, 0}},
+//  {"left_ptr",		IMAGEBITMAP, MacXCursorData(left_ptr), nil, {0, 0}},
+    {"left_side",		IMAGEBITMAP, MacXCursorData(left_side), nil, {0, 0}},
+    {"left_tee",		IMAGEBITMAP, MacXCursorData(left_tee), nil, {0, 0}},
+    {"leftbutton",		IMAGEBITMAP, MacXCursorData(leftbutton), nil, {0, 0}},
+    {"ll_angle",		IMAGEBITMAP, MacXCursorData(ll_angle), nil, {0, 0}},
+    {"lr_angle",		IMAGEBITMAP, MacXCursorData(lr_angle), nil, {0, 0}},
+    {"man",			IMAGEBITMAP, MacXCursorData(man), nil, {0, 0}},
+    {"middlebutton",		IMAGEBITMAP, MacXCursorData(middlebutton), nil, {0, 0}},
+    {"mouse",			IMAGEBITMAP, MacXCursorData(mouse), nil, {0, 0}},
+    {"pencil",			IMAGEBITMAP, MacXCursorData(pencil), nil, {0, 0}},
+    {"pirate",			IMAGEBITMAP, MacXCursorData(pirate), nil, {0, 0}},
+    {"plus",			IMAGEBITMAP, MacXCursorData(plus), nil, {0, 0}},
+    {"question_arrow",		IMAGEBITMAP, MacXCursorData(question_arrow), nil, {0, 0}},
+    {"right_ptr",		IMAGEBITMAP, MacXCursorData(right_ptr), nil, {0, 0}},
+    {"right_side",		IMAGEBITMAP, MacXCursorData(right_side), nil, {0, 0}},
+    {"right_tee",		IMAGEBITMAP, MacXCursorData(right_tee), nil, {0, 0}},
+    {"rightbutton",		IMAGEBITMAP, MacXCursorData(rightbutton), nil, {0, 0}},
+    {"rtl_logo",		IMAGEBITMAP, MacXCursorData(rtl_logo), nil, {0, 0}},
+    {"sailboat",		IMAGEBITMAP, MacXCursorData(sailboat), nil, {0, 0}},
+    {"sb_down_arrow",		IMAGEBITMAP, MacXCursorData(sb_down_arrow), nil, {0, 0}},
+    {"sb_h_double_arrow",	IMAGEBITMAP, MacXCursorData(sb_h_double_arrow), nil, {0, 0}},
+    {"sb_left_arrow",		IMAGEBITMAP, MacXCursorData(sb_left_arrow), nil, {0, 0}},
+    {"sb_right_arrow",		IMAGEBITMAP, MacXCursorData(sb_right_arrow), nil, {0, 0}},
+    {"sb_up_arrow",		IMAGEBITMAP, MacXCursorData(sb_up_arrow), nil, {0, 0}},
+    {"sb_v_double_arrow",	IMAGEBITMAP, MacXCursorData(sb_v_double_arrow), nil, {0, 0}},
+    {"shuttle",			IMAGEBITMAP, MacXCursorData(shuttle), nil, {0, 0}},
+    {"sizing",			IMAGEBITMAP, MacXCursorData(sizing), nil, {0, 0}},
+    {"spider",			IMAGEBITMAP, MacXCursorData(spider), nil, {0, 0}},
+    {"spraycan",		IMAGEBITMAP, MacXCursorData(spraycan), nil, {0, 0}},
+    {"star",			IMAGEBITMAP, MacXCursorData(star), nil, {0, 0}},
+    {"target",			IMAGEBITMAP, MacXCursorData(target), nil, {0, 0}},
+//  {"tcross",			IMAGEBITMAP, MacXCursorData(tcross), nil, {0, 0}},
+//  {"top_left_arrow",		IMAGEBITMAP, MacXCursorData(top_left_arrow), nil, {0, 0}},
+    {"top_left_corner",		IMAGEBITMAP, MacXCursorData(top_left_corner), nil, {0, 0}},
+    {"top_right_corner",	IMAGEBITMAP, MacXCursorData(top_right_corner), nil, {0, 0}},
+    {"top_side",		IMAGEBITMAP, MacXCursorData(top_side), nil, {0, 0}},
+    {"top_tee",			IMAGEBITMAP, MacXCursorData(top_tee), nil, {0, 0}},
+    {"trek",			IMAGEBITMAP, MacXCursorData(trek), nil, {0, 0}},
+    {"ul_angle",		IMAGEBITMAP, MacXCursorData(ul_angle), nil, {0, 0}},
+    {"umbrella",		IMAGEBITMAP, MacXCursorData(umbrella), nil, {0, 0}},
+    {"ur_angle",		IMAGEBITMAP, MacXCursorData(ur_angle), nil, {0, 0}},
+    {"watch",			IMAGEBITMAP, MacXCursorData(watch), nil, {0, 0}},
+//  {"xterm",			IMAGEBITMAP, MacXCursorData(xterm), nil, {0, 0}},
+    {NULL, 0, nil, nil, {0, 0}}
 };
 
 /*
@@ -271,7 +271,7 @@ FindCursorByName(
 		break;
 	    case IMAGEBITMAP: {
 		unsigned char *bitmap = (unsigned char *)(cursorNames[idx].id1);
-		NSBitmapImageRep *bitmapImageRep = NULL;
+		NSBitmapImageRep *bitmapImageRep = nil;
 		CGImageRef img = NULL, mask = NULL, maskedImg = NULL;
 		static const CGFloat decodeWB[] = {1, 0};
 		CGColorSpaceRef colorspace = CGColorSpaceCreateWithName(
@@ -312,6 +312,7 @@ FindCursorByName(
 		if (bitmapImageRep) {
 		    image = [[NSImage alloc] initWithSize:NSMakeSize(pix, pix)];
 		    [image addRepresentation:bitmapImageRep];
+		    [image setTemplate:YES];
 		    [bitmapImageRep release];
 		}
 
@@ -374,7 +375,7 @@ FindCursorByName(
 TkCursor *
 TkGetCursorByName(
     Tcl_Interp *interp,		/* Interpreter to use for error reporting. */
-    Tk_Window tkwin,		/* Window in which cursor will be used. */
+    TCL_UNUSED(Tk_Window),		/* Window in which cursor will be used. */
     Tk_Uid string)		/* Description of cursor. See manual entry
 				 * for details on legal syntax. */
 {
@@ -389,7 +390,7 @@ TkGetCursorByName(
 
     if (Tcl_SplitList(interp, string, &argc, &argv) == TCL_OK) {
 	if (argc) {
-	    macCursorPtr = ckalloc(sizeof(TkMacOSXCursor));
+	    macCursorPtr = (TkMacOSXCursor *)ckalloc(sizeof(TkMacOSXCursor));
 	    macCursorPtr->info.cursor = (Tk_Cursor) macCursorPtr;
 	    macCursorPtr->macCursor = nil;
 	    macCursorPtr->type = 0;
@@ -428,13 +429,15 @@ TkGetCursorByName(
 
 TkCursor *
 TkCreateCursorFromData(
-    Tk_Window tkwin,		/* Window in which cursor will be used. */
-    const char *source,		/* Bitmap data for cursor shape. */
-    const char *mask,		/* Bitmap data for cursor mask. */
-    int width, int height,	/* Dimensions of cursor. */
-    int xHot, int yHot,		/* Location of hot-spot in cursor. */
-    XColor fgColor,		/* Foreground color for cursor. */
-    XColor bgColor)		/* Background color for cursor. */
+    TCL_UNUSED(Tk_Window),		/* Window in which cursor will be used. */
+    TCL_UNUSED(const char *),		/* Bitmap data for cursor shape. */
+    TCL_UNUSED(const char *),		/* Bitmap data for cursor mask. */
+    TCL_UNUSED(int),	/* Dimensions of cursor. */
+    TCL_UNUSED(int),
+    TCL_UNUSED(int),		/* Location of hot-spot in cursor. */
+    TCL_UNUSED(int),
+    TCL_UNUSED(XColor),		/* Foreground color for cursor. */
+    TCL_UNUSED(XColor))		/* Background color for cursor. */
 {
     return NULL;
 }
@@ -463,7 +466,7 @@ TkpFreeCursor(
     TkMacOSXCursor *macCursorPtr = (TkMacOSXCursor *) cursorPtr;
 
     [macCursorPtr->macCursor release];
-    macCursorPtr->macCursor = NULL;
+    macCursorPtr->macCursor = nil;
     if (macCursorPtr == gCurrentCursor) {
 	gCurrentCursor = NULL;
     }
@@ -548,7 +551,7 @@ TkpSetCursor(
 	return;
     }
 
-    if (cursor == None) {
+    if (cursor == NULL) {
 	/*
 	 * This is a little tricky. We can't really tell whether
 	 * gCurrentCursor is NULL because it was NULL last time around or
