@@ -12,13 +12,18 @@
 
 #include "tkUnixInt.h"
 #include <signal.h>
+#ifdef HAVE_XKBKEYCODETOKEYSYM
+#  include <X11/XKBlib.h>
+#else
+#  define XkbOpenDisplay(D,V,E,M,m,R) (((void)D),((void)V),((void)E),((void)M),((void)m),((void)R),(NULL))
+#endif
 
 /*
  * The following static indicates whether this module has been initialized in
  * the current thread.
  */
 
-typedef struct ThreadSpecificData {
+typedef struct {
     int initialized;
 } ThreadSpecificData;
 static Tcl_ThreadDataKey dataKey;
@@ -33,6 +38,8 @@ static void		DisplayFileProc(ClientData clientData, int flags);
 static void		DisplaySetupProc(ClientData clientData, int flags);
 static void		TransferXEventsToTcl(Display *display);
 #ifdef TK_USE_INPUT_METHODS
+static void		InstantiateIMCallback(Display *, XPointer client_data, XPointer call_data);
+static void		DestroyIMCallback(XIM im, XPointer client_data, XPointer call_data);
 static void		OpenIM(TkDisplay *dispPtr);
 #endif
 
@@ -116,7 +123,54 @@ TkpOpenDisplay(
     const char *displayNameStr)
 {
     TkDisplay *dispPtr;
-    Display *display = XOpenDisplay(displayNameStr);
+    Display *display;
+    int event = 0;
+    int error = 0;
+    int major = 1;
+    int minor = 0;
+    int reason = 0;
+    unsigned int use_xkb = 0;
+    /* Disabled, until we have a better test. See [Bug 3613668] */
+#if 0 && defined(XKEYCODETOKEYSYM_IS_DEPRECATED) && defined(TCL_THREADS)
+    static int xinited = 0;
+    static Tcl_Mutex xinitMutex = NULL;
+
+    if (!xinited) {
+	Tcl_MutexLock(&xinitMutex);
+	if (!xinited) {
+	    /* Necessary for threaded apps, of no consequence otherwise  */
+	    /* need only be called once, but must be called before *any* */
+	    /* Xlib call is made. If xinitMutex is still NULL after the  */
+	    /* Tcl_MutexLock call, Tcl was compiled without threads so   */
+	    /* we cannot use XInitThreads() either.                      */
+	    if (xinitMutex != NULL){
+		XInitThreads();
+	    }
+	    xinited = 1;
+	}
+	Tcl_MutexUnlock(&xinitMutex);
+    }
+#endif
+
+    /*
+    ** Bug [3607830]: Before using Xkb, it must be initialized and confirmed
+    **                that the serve supports it.  The XkbOpenDisplay call
+    **                will perform this check and return NULL if the extension
+    **                is not supported.
+    **
+    ** Work around un-const-ified Xkb headers using (char *) cast.
+    */
+    display = XkbOpenDisplay((char *)displayNameStr, &event, &error, &major,
+	    &minor, &reason);
+
+    if (display == NULL) {
+	/*fprintf(stderr,"event=%d error=%d major=%d minor=%d reason=%d\nDisabling xkb\n",
+	event, error, major, minor, reason);*/
+	display = XOpenDisplay(displayNameStr);
+    } else {
+	use_xkb = TK_DISPLAY_USE_XKB;
+	/*fprintf(stderr, "Using xkb %d.%d\n", major, minor);*/
+    }
 
     if (display == NULL) {
 	return NULL;
@@ -124,11 +178,40 @@ TkpOpenDisplay(
     dispPtr = ckalloc(sizeof(TkDisplay));
     memset(dispPtr, 0, sizeof(TkDisplay));
     dispPtr->display = display;
+    dispPtr->flags |= use_xkb;
 #ifdef TK_USE_INPUT_METHODS
+    XSetLocaleModifiers("");
     OpenIM(dispPtr);
+    XRegisterIMInstantiateCallback(dispPtr->display, NULL, NULL, NULL,
+	    InstantiateIMCallback, (XPointer) dispPtr);
 #endif
     Tcl_CreateFileHandler(ConnectionNumber(display), TCL_READABLE,
 	    DisplayFileProc, dispPtr);
+
+    /*
+     * Observed weird WidthMMOfScreen() in X on Wayland on a
+     * Fedora 30/i386 running in a VM. Fallback to 75 dpi,
+     * otherwise many other strange things may happen later.
+     * See: [https://core.tcl-lang.org/tk/tktview?name=a01b6f7227]
+     */
+    if (WidthMMOfScreen(DefaultScreenOfDisplay(display)) <= 0) {
+	int mm;
+
+	mm = WidthOfScreen(DefaultScreenOfDisplay(display)) * (25.4 / 75.0);
+	WidthMMOfScreen(DefaultScreenOfDisplay(display)) = mm;
+    }
+    if (HeightMMOfScreen(DefaultScreenOfDisplay(display)) <= 0) {
+	int mm;
+
+	mm = HeightOfScreen(DefaultScreenOfDisplay(display)) * (25.4 / 75.0);
+	HeightMMOfScreen(DefaultScreenOfDisplay(display)) = mm;
+    }
+
+    /*
+     * Key map info must be available immediately, because of "send event".
+     */
+    TkpInitKeymapInfo(dispPtr);
+
     return dispPtr;
 }
 
@@ -278,9 +361,6 @@ TransferXEventsToTcl(
 	int type;
 	XEvent x;
 	TkKeyEvent k;
-#ifdef GenericEvent
-	xGenericEvent xge;
-#endif
     } event;
     Window w;
     TkDisplay *dispPtr = NULL;
@@ -298,12 +378,11 @@ TransferXEventsToTcl(
 
     while (QLength(display) > 0) {
 	XNextEvent(display, &event.x);
-#ifdef GenericEvent
-	if (event.type == GenericEvent) {
-	    Tcl_Panic("Wild GenericEvent; panic! (extension=%d,evtype=%d)",
-		    event.xge.extension, event.xge.evtype);
+	if ((event.type >= VirtualEvent) && (event.type <= MouseWheelEvent)) {
+	    /* See [fe87e9af39]. Those are internal Tk event types, if they come
+	     * from an external source they most likely would be totally mis-interpreted */
+	    continue;
 	}
-#endif
 	w = None;
 	if (event.type == KeyPress || event.type == KeyRelease) {
 	    for (dispPtr = TkGetDisplayList(); ; dispPtr = dispPtr->nextPtr) {
@@ -432,9 +511,9 @@ DisplayFileProc(
 	 * nice (?!) message.
 	 */
 
-	void (*oldHandler)();
+	void (*oldHandler)(int);
 
-	oldHandler = (void (*)()) signal(SIGPIPE, SIG_IGN);
+	oldHandler = (void (*)(int)) signal(SIGPIPE, SIG_IGN);
 	XNoOp(display);
 	XFlush(display);
 	(void) signal(SIGPIPE, oldHandler);
@@ -611,6 +690,35 @@ TkpSync(
 }
 #ifdef TK_USE_INPUT_METHODS
 
+static void
+InstantiateIMCallback(
+    Display      *display,
+    XPointer     client_data,
+    XPointer     call_data)
+{
+    TkDisplay    *dispPtr;
+
+    dispPtr = (TkDisplay *) client_data;
+    OpenIM(dispPtr);
+    XUnregisterIMInstantiateCallback(dispPtr->display, NULL, NULL, NULL,
+	    InstantiateIMCallback, (XPointer) dispPtr);
+}
+
+static void
+DestroyIMCallback(
+    XIM         im,
+    XPointer    client_data,
+    XPointer    call_data)
+{
+    TkDisplay   *dispPtr;
+
+    dispPtr = (TkDisplay *) client_data;
+    dispPtr->inputMethod = NULL;
+    ++dispPtr->ximGeneration;
+    XRegisterIMInstantiateCallback(dispPtr->display, NULL, NULL, NULL,
+	    InstantiateIMCallback, (XPointer) dispPtr);
+}
+
 /*
  *--------------------------------------------------------------
  *
@@ -636,15 +744,23 @@ OpenIM(
     XIMStyles *stylePtr;
     XIMStyle bestStyle = 0;
 
-    if (XSetLocaleModifiers("") == NULL) {
-	return;
-    }
-
+    ++dispPtr->ximGeneration;
     dispPtr->inputMethod = XOpenIM(dispPtr->display, NULL, NULL, NULL);
     if (dispPtr->inputMethod == NULL) {
 	return;
     }
 
+    /* Require X11R6 */
+    {
+	XIMCallback destroy_cb;
+
+	destroy_cb.callback = DestroyIMCallback;
+	destroy_cb.client_data = (XPointer) dispPtr;
+	if (XSetIMValues(dispPtr->inputMethod, XNDestroyCallback,
+		&destroy_cb, NULL))
+	    goto error;
+    }
+
     if ((XGetIMValues(dispPtr->inputMethod, XNQueryInputStyle, &stylePtr,
 	    NULL) != NULL) || (stylePtr == NULL)) {
 	goto error;
@@ -691,6 +807,7 @@ error:
     if (dispPtr->inputMethod) {
 	XCloseIM(dispPtr->inputMethod);
 	dispPtr->inputMethod = NULL;
+	++dispPtr->ximGeneration;
     }
 }
 #endif /* TK_USE_INPUT_METHODS */
