@@ -59,6 +59,7 @@
 #define DEF_BUTTON_DEFAULT		"disabled"
 #define DEF_BUTTON_DISABLED_FG_COLOR	DISABLED
 #define DEF_BUTTON_DISABLED_FG_MONO	""
+#define DEF_LABEL_FG			NORMAL_FG
 #define DEF_BUTTON_FG			NORMAL_FG
 #define DEF_CHKRAD_FG			TEXT_FG
 #define DEF_BUTTON_FONT			"TkDefaultFont"
@@ -489,7 +490,7 @@
 #define DEF_TEXT_PADX			"1"
 #define DEF_TEXT_PADY			"1"
 #define DEF_TEXT_RELIEF			"sunken"
-#define DEF_TEXT_INACTIVE_SELECT_COLOR	NULL
+#define DEF_TEXT_INACTIVE_SELECT_BG_COLOR	NULL
 #define DEF_TEXT_SELECT_COLOR		SELECT_BG
 #define DEF_TEXT_SELECT_MONO		BLACK
 #define DEF_TEXT_SELECT_BD_COLOR	"0"
