@@ -49,7 +49,11 @@ static TkpClipMask *AllocClipMask(GC gc) {
     if (clip_mask == NULL) {
 	clip_mask = (TkpClipMask *)ckalloc(sizeof(TkpClipMask));
 	gc->clip_mask = (Pixmap) clip_mask;
+    } else if (clip_mask->type == TKP_CLIP_REGION) {
+	TkDestroyRegion(clip_mask->value.region);
     }
+    clip_mask->type = TKP_CLIP_PIXMAP;
+    clip_mask->value.pixmap = None;
     return clip_mask;
 }
 
@@ -70,10 +74,15 @@ static TkpClipMask *AllocClipMask(GC gc) {
  */
 
 static void FreeClipMask(GC gc) {
-    if (gc->clip_mask != None) {
-	ckfree((char *)gc->clip_mask);
-	gc->clip_mask = None;
+    TkpClipMask * clip_mask = (TkpClipMask*)gc->clip_mask;
+    if (clip_mask == NULL) {
+	return;
     }
+    if (clip_mask->type == TKP_CLIP_REGION) {
+	TkDestroyRegion(clip_mask->value.region);
+    }
+    ckfree(clip_mask);
+    gc->clip_mask = None;
 }
 
 /*
@@ -423,14 +432,12 @@ XSetClipOrigin(
 /*
  *----------------------------------------------------------------------
  *
- * TkSetRegion, XSetClipMask --
+ * TkSetRegion, XSetClipMask, XSetClipRectangles --
  *
  *	Sets the clipping region/pixmap for a GC.
  *
- *	Note that unlike the Xlib equivalent, it is not safe to delete the
- *	region after setting it into the GC (except on Mac OS X). The only
- *	uses of TkSetRegion are currently in DisplayFrame and in
- *	ImgPhotoDisplay, which use the GC immediately.
+ *	Like the Xlib equivalent, it is safe to delete the
+ *	region after setting it into the GC.
  *
  * Results:
  *	None.
@@ -456,6 +463,8 @@ TkSetRegion(
 
 	clip_mask->type = TKP_CLIP_REGION;
 	clip_mask->value.region = r;
+	clip_mask->value.region = TkCreateRegion();
+	TkpCopyRegion(clip_mask->value.region, r);
     }
     return Success;
 }
@@ -479,6 +488,32 @@ XSetClipMask(
     return Success;
 }
 
+int
+XSetClipRectangles(
+    TCL_UNUSED(Display*),
+    GC gc,
+    int clip_x_origin,
+    int clip_y_origin,
+    XRectangle* rectangles,
+    int n,
+    TCL_UNUSED(int))
+{
+    TkRegion clipRgn = TkCreateRegion();
+    TkpClipMask * clip_mask = AllocClipMask(gc);
+    clip_mask->type = TKP_CLIP_REGION;
+    clip_mask->value.region = clipRgn;
+
+    while (n--) {
+	XRectangle rect = *rectangles;
+
+	rect.x += clip_x_origin;
+	rect.y += clip_y_origin;
+	TkUnionRectWithRegion(&rect, clipRgn, clipRgn);
+	rectangles++;
+    }
+    return 1;
+}
+
 /*
  * Some additional dummy functions (hopefully implemented soon).
  */
